@@ -1,4 +1,4 @@
-# Copyright 2016 IBM Corp.
+# Copyright 2016, 2017 IBM Corp.
 #
 # All Rights Reserved.
 #
@@ -22,7 +22,7 @@ from nova_powervm import conf as cfg
 CONF = cfg.CONF
 
 
-class TestConf(test.TestCase):
+class TestConf(test.NoDBTestCase):
     def setUp(self):
         super(TestConf, self).setUp()
 
@@ -42,7 +42,7 @@ class TestConf(test.TestCase):
         self.assertEqual(1, CONF.powervm.ports_per_fabric)
 
 
-class TestConfBounds(test.TestCase):
+class TestConfBounds(test.NoDBTestCase):
     def setUp(self):
         super(TestConfBounds, self).setUp()
 
@@ -89,7 +89,7 @@ class TestConfBounds(test.TestCase):
                           ports_per_fabric=2)
 
 
-class TestConfChoices(test.TestCase):
+class TestConfChoices(test.NoDBTestCase):
     def setUp(self):
         super(TestConfChoices, self).setUp()
 
@@ -133,15 +133,12 @@ class TestConfChoices(test.TestCase):
                 fx.conf.powervm[option]
 
     def test_choices(self):
-        # Disk driver
-        self._choice_test('bad_driver', ['localdisk', 'ssp'],
-                          cfg.powervm.powervm_opts, 'disk_driver')
         # FC attachment
         self._choice_test('bad_value', ['vscsi', 'npiv'],
                           cfg.powervm.vol_adapter_opts, 'fc_attach_strategy')
 
 
-class TestConfDynamic(test.TestCase):
+class TestConfDynamic(test.NoDBTestCase):
     def setUp(self):
         super(TestConfDynamic, self).setUp()
         self.conf_fx = self.useFixture(
