@@ -1,4 +1,4 @@
-# Copyright 2015, 2017 IBM Corp.
+# Copyright IBM Corp. and contributors
 #
 # All Rights Reserved.
 #
@@ -19,6 +19,7 @@ import mock
 
 from nova import exception as nova_exc
 from nova import test
+from oslo_utils.fixture import uuidsentinel as uuids
 from pypowervm import const as pvm_const
 from pypowervm.tasks import storage as tsk_stor
 from pypowervm.tests import test_fixtures as pvm_fx
@@ -33,7 +34,7 @@ from nova_powervm.virt.powervm import exception as npvmex
 from nova_powervm.virt.powervm import vm
 
 
-class TestLocalDisk(test.TestCase):
+class TestLocalDisk(test.NoDBTestCase):
     """Unit Tests for the LocalDisk storage driver."""
 
     def setUp(self):
@@ -41,15 +42,15 @@ class TestLocalDisk(test.TestCase):
 
         self.apt = self.useFixture(pvm_fx.AdapterFx()).adpt
 
-        # Set up mock for internal VIOS.get()s
-        self.mock_vios_get = self.useFixture(fixtures.MockPatch(
-            'pypowervm.wrappers.virtual_io_server.VIOS',
-            autospec=True)).mock.get
         # The mock VIOS needs to have scsi_mappings as a list.  Internals are
         # set by individual test cases as needed.
         smaps = [mock.Mock()]
         self.vio_to_vg = mock.Mock(spec=pvm_vios.VIOS, scsi_mappings=smaps,
                                    uuid='vios-uuid')
+        # Set up mock for internal VIOS.get()s
+        self.mock_vios_get = self.useFixture(fixtures.MockPatch(
+            'pypowervm.wrappers.virtual_io_server.VIOS',
+            autospec=True)).mock.get
         # For our tests, we want find_maps to return the mocked list of scsi
         # mappings in our mocked VIOS.
         self.mock_find_maps = self.useFixture(fixtures.MockPatch(
@@ -59,7 +60,7 @@ class TestLocalDisk(test.TestCase):
         # Set up for the mocks for get_ls
         self.mock_find_vg = self.useFixture(fixtures.MockPatch(
             'pypowervm.tasks.storage.find_vg', autospec=True)).mock
-        self.vg_uuid = 'd5065c2c-ac43-3fa6-af32-ea84a3960291'
+        self.vg_uuid = uuids.vg_uuid
         self.vg = mock.Mock(spec=pvm_stor.VG, uuid=self.vg_uuid)
         self.mock_find_vg.return_value = (self.vio_to_vg, self.vg)
 
@@ -68,7 +69,7 @@ class TestLocalDisk(test.TestCase):
             'nova_powervm.virt.powervm.mgmt.mgmt_uuid')).mock
         self.mgmt_uuid.return_value = 'mp_uuid'
 
-        self.flags(volume_group_name='rootvg', group='powervm')
+        self.flags(volume_group_name='fakevg', group='powervm')
 
     @staticmethod
     def get_ls(adpt):
@@ -76,7 +77,7 @@ class TestLocalDisk(test.TestCase):
 
     def test_init(self):
         local = self.get_ls(self.apt)
-        self.mock_find_vg.assert_called_once_with(self.apt, 'rootvg')
+        self.mock_find_vg.assert_called_once_with(self.apt, 'fakevg')
         self.assertEqual('vios-uuid', local._vios_uuid)
         self.assertEqual(self.vg_uuid, local.vg_uuid)
         self.assertEqual(self.apt, local.adapter)
@@ -102,10 +103,12 @@ class TestLocalDisk(test.TestCase):
         mock_get_image.reset_mock()
         exception = Exception
         mock_get_image.side_effect = exception
-        self.assertRaises(exception,
-                          self.get_ls(self.apt).create_disk_from_image,
-                          None, inst, powervm.TEST_IMAGE1)
+        with mock.patch('time.sleep', autospec=True) as mock_sleep:
+            self.assertRaises(exception,
+                              self.get_ls(self.apt).create_disk_from_image,
+                              None, inst, powervm.TEST_IMAGE1)
         self.assertEqual(mock_get_image.call_count, 4)
+        self.assertEqual(3, mock_sleep.call_count)
 
     @mock.patch('pypowervm.tasks.storage.upload_new_vdisk', autospec=True)
     @mock.patch('nova.image.api.API.download')
@@ -149,19 +152,13 @@ class TestLocalDisk(test.TestCase):
         mock_img_api.assert_not_called()
         self.assertEqual(0, mock_upload_vdisk.call_count)
 
-    @mock.patch('pypowervm.wrappers.storage.VG', autospec=True)
+    @mock.patch('nova_powervm.virt.powervm.disk.localdisk.LocalStorage.'
+                '_get_vg_wrap')
     def test_capacity(self, mock_vg):
         """Tests the capacity methods."""
-
-        # Set up the mock data.  This will simulate our vg wrapper
-        mock_vg_wrap = mock.MagicMock(name='vg_wrapper')
-        type(mock_vg_wrap).capacity = mock.PropertyMock(return_value='5120')
-        type(mock_vg_wrap).available_size = mock.PropertyMock(
-            return_value='2048')
-
-        mock_vg.get.return_value = mock_vg_wrap
         local = self.get_ls(self.apt)
-
+        mock_vg.return_value = mock.Mock(
+            capacity='5120', available_size='2048')
         self.assertEqual(5120.0, local.capacity)
         self.assertEqual(3072.0, local.capacity_used)
 
@@ -353,14 +350,15 @@ class TestLocalDisk(test.TestCase):
     def _bld_mocks_for_instance_disk(self):
         inst = mock.Mock()
         inst.name = 'Name Of Instance'
-        inst.uuid = 'd5065c2c-ac43-3fa6-af32-ea84a3960291'
+        inst.uuid = uuids.inst_uuid
         lpar_wrap = mock.Mock()
         lpar_wrap.id = 2
         vios1 = self.vio_to_vg
-        vios1.scsi_mappings[0].backing_storage.name = 'b_Name_Of__d506'
+        back_stor_name = 'b_Name_Of__' + inst.uuid[:4]
+        vios1.scsi_mappings[0].backing_storage.name = back_stor_name
         return inst, lpar_wrap, vios1
 
-    def test_boot_disk_path_for_instance(self):
+    def test_get_bootdisk_path(self):
         local = self.get_ls(self.apt)
         inst = mock.Mock()
         inst.name = 'Name Of Instance'
@@ -369,17 +367,20 @@ class TestLocalDisk(test.TestCase):
         vios1.scsi_mappings[0].server_adapter.backing_dev_name = 'boot_7f81628'
         vios1.scsi_mappings[0].backing_storage.name = 'b_Name_Of__f921'
         self.mock_vios_get.return_value = vios1
-        dev_name = local.boot_disk_path_for_instance(inst, vios1.uuid)
+        dev_name = local.get_bootdisk_path(inst, vios1.uuid)
         self.assertEqual('boot_7f81628', dev_name)
 
+    @mock.patch('nova_powervm.virt.powervm.vm.get_instance_wrapper',
+                autospec=True)
     @mock.patch('pypowervm.wrappers.storage.VG.get', new=mock.Mock())
-    def test_instance_disk_iter(self):
+    def test_get_bootdisk_iter(self, mock_lpar_wrap):
         local = self.get_ls(self.apt)
         inst, lpar_wrap, vios1 = self._bld_mocks_for_instance_disk()
+        mock_lpar_wrap.return_value = lpar_wrap
 
         # Good path
         self.mock_vios_get.return_value = vios1
-        for vdisk, vios in local.instance_disk_iter(inst, lpar_wrap=lpar_wrap):
+        for vdisk, vios in local._get_bootdisk_iter(inst):
             self.assertEqual(vios1.scsi_mappings[0].backing_storage, vdisk)
             self.assertEqual(vios1.uuid, vios.uuid)
         self.mock_vios_get.assert_called_once_with(
@@ -388,8 +389,8 @@ class TestLocalDisk(test.TestCase):
         # Not found because no storage of that name
         self.mock_vios_get.reset_mock()
         self.mock_find_maps.return_value = []
-        for vdisk, vios in local.instance_disk_iter(inst, lpar_wrap=lpar_wrap):
-            self.fail()
+        for vdisk, vios in local._get_bootdisk_iter(inst):
+            self.fail('Should not have found any storage elements.')
         self.mock_vios_get.assert_called_once_with(
             self.apt, uuid='vios-uuid', xag=[pvm_const.XAG.VIO_SMAP])
 
@@ -429,3 +430,18 @@ class TestLocalDisk(test.TestCase):
         local.disconnect_disk_from_mgmt('vios-uuid', 'disk_name')
         mock_rm_vdisk_map.assert_called_with(
             local.adapter, 'vios-uuid', 'mp_uuid', disk_names=['disk_name'])
+
+    def test_capabilities_non_mgmt_vios(self):
+        local = self.get_ls(self.apt)
+        self.assertFalse(local.capabilities.get('shared_storage'))
+        self.assertTrue(local.capabilities.get('has_imagecache'))
+        # With the default setup, the management partition isn't the VIOS.
+        self.assertFalse(local.capabilities.get('snapshot'))
+
+    def test_capabilities_mgmt_vios(self):
+        # Make the management partition the VIOS.
+        self.vio_to_vg.uuid = self.mgmt_uuid.return_value
+        local = self.get_ls(self.apt)
+        self.assertFalse(local.capabilities.get('shared_storage'))
+        self.assertTrue(local.capabilities.get('has_imagecache'))
+        self.assertTrue(local.capabilities.get('snapshot'))
