@@ -1,4 +1,4 @@
-# Copyright 2016 IBM Corp.
+# Copyright 2016, 2018 IBM Corp.
 #
 # All Rights Reserved.
 #
@@ -25,7 +25,7 @@ from nova_powervm.virt.powervm.nvram import api
 from nova_powervm.virt.powervm.nvram import swift
 
 
-class TestSwiftStore(test.TestCase):
+class TestSwiftStore(test.NoDBTestCase):
 
     def setUp(self):
         super(TestSwiftStore, self).setUp()
@@ -89,7 +89,7 @@ class TestSwiftStore(test.TestCase):
                                              options={'long': True})
 
     @mock.patch('nova_powervm.virt.powervm.nvram.swift.SwiftNvramStore.'
-                '_get_container_names')
+                '_get_container_names', autospec=True)
     def test_get_object_names(self, mock_container_names):
         with mock.patch.object(self.swift_store, '_run_operation') as mock_run:
             mock_run.return_value = self._build_results(['obj', 'obj2'])
@@ -120,15 +120,14 @@ class TestSwiftStore(test.TestCase):
             # Second run should not increment the call count here
             self.assertEqual(mock_container_names.call_count, 2)
 
-    @mock.patch('swiftclient.service.SwiftUploadObject')
+    @mock.patch('swiftclient.service.SwiftUploadObject', autospec=True)
     @mock.patch('nova_powervm.virt.powervm.nvram.swift.SwiftNvramStore.'
-                '_exists')
+                '_exists', autospec=True)
     def test_underscore_store(self, mock_exists, mock_swiftuploadobj):
         mock_exists.return_value = True
         with mock.patch.object(self.swift_store, '_run_operation') as mock_run:
             mock_run.return_value = self._build_results(['obj'])
-            self.swift_store._store(powervm.TEST_INST1.uuid,
-                                    powervm.TEST_INST1.name, 'data')
+            self.swift_store._store(powervm.TEST_INST1.uuid, 'data')
             mock_run.assert_called_once_with('upload', 'powervm_nvram',
                                              mock.ANY, options=None)
 
@@ -138,7 +137,7 @@ class TestSwiftStore(test.TestCase):
             mock_run.return_value = mock_result
             self.assertRaises(api.NVRAMUploadException,
                               self.swift_store._store, powervm.TEST_INST1.uuid,
-                              powervm.TEST_INST1.name, 'data')
+                              'data')
 
             # Test retry upload
             mock_run.reset_mock()
@@ -149,23 +148,21 @@ class TestSwiftStore(test.TestCase):
                             'object': '6ecb1386-53ab-43da-9e04-54e986ad4a9d'}
             mock_run.side_effect = [mock_res_obj,
                                     self._build_results(['obj'])]
-            self.swift_store._store(powervm.TEST_INST1.uuid,
-                                    powervm.TEST_INST1.name, 'data')
+            self.swift_store._store(powervm.TEST_INST1.uuid, 'data')
             mock_run.assert_called_with('upload', 'powervm_nvram',
                                         mock.ANY, options=None)
             self.assertEqual(mock_run.call_count, 2)
             self.assertEqual(mock_swiftuploadobj.call_count, 2)
 
-    @mock.patch('swiftclient.service.SwiftUploadObject')
+    @mock.patch('swiftclient.service.SwiftUploadObject', autospec=True)
     @mock.patch('nova_powervm.virt.powervm.nvram.swift.SwiftNvramStore.'
-                '_exists')
+                '_exists', autospec=True)
     def test_underscore_store_not_exists(self, mock_exists,
                                          mock_swiftuploadobj):
         mock_exists.return_value = False
         with mock.patch.object(self.swift_store, '_run_operation') as mock_run:
             mock_run.return_value = self._build_results(['obj'])
-            self.swift_store._store(powervm.TEST_INST1.uuid,
-                                    powervm.TEST_INST1.name, 'data')
+            self.swift_store._store(powervm.TEST_INST1.uuid, 'data')
             mock_run.assert_called_once_with(
                 'upload', 'powervm_nvram', mock.ANY,
                 options={'leave_segments': True})
@@ -179,22 +176,20 @@ class TestSwiftStore(test.TestCase):
                             'object': '6ecb1386-53ab-43da-9e04-54e986ad4a9d'}
             mock_run.side_effect = [mock_res_obj,
                                     self._build_results(['obj'])]
-            self.swift_store._store(powervm.TEST_INST1.uuid,
-                                    powervm.TEST_INST1.name, 'data')
+            self.swift_store._store(powervm.TEST_INST1.uuid, 'data')
             mock_run.assert_called_with('upload', 'powervm_nvram', mock.ANY,
                                         options={'leave_segments': True})
             self.assertEqual(mock_run.call_count, 2)
             self.assertEqual(mock_swiftuploadobj.call_count, 2)
 
     @mock.patch('nova_powervm.virt.powervm.nvram.swift.SwiftNvramStore.'
-                '_exists')
+                '_exists', autospec=True)
     def test_store(self, mock_exists):
         # Test forcing a update
         with mock.patch.object(self.swift_store, '_store') as mock_store:
             mock_exists.return_value = False
-            self.swift_store.store(powervm.TEST_INST1, 'data', force=True)
+            self.swift_store.store(powervm.TEST_INST1.uuid, 'data', force=True)
             mock_store.assert_called_once_with(powervm.TEST_INST1.uuid,
-                                               powervm.TEST_INST1.name,
                                                'data', exists=False)
 
         with mock.patch.object(
@@ -206,7 +201,8 @@ class TestSwiftStore(test.TestCase):
             results = self._build_results(['obj'])
             results[0]['headers'] = {'etag': data_md5_hash}
             mock_run.return_value = results
-            self.swift_store.store(powervm.TEST_INST1, 'data', force=False)
+            self.swift_store.store(powervm.TEST_INST1.uuid, 'data',
+                                   force=False)
             self.assertFalse(mock_store.called)
             mock_run.assert_called_once_with(
                 'stat', options={'long': True},
@@ -217,12 +213,12 @@ class TestSwiftStore(test.TestCase):
         with mock.patch.object(self.swift_store, '_store') as mock_store:
             self.swift_store.store_slot_map("test_slot", 'data')
             mock_store.assert_called_once_with(
-                'test_slot', 'test_slot', 'data')
+                'test_slot', 'data')
 
-    @mock.patch('os.remove')
-    @mock.patch('tempfile.NamedTemporaryFile')
+    @mock.patch('os.remove', autospec=True)
+    @mock.patch('tempfile.NamedTemporaryFile', autospec=True)
     @mock.patch('nova_powervm.virt.powervm.nvram.swift.SwiftNvramStore.'
-                '_exists')
+                '_exists', autospec=True)
     def test_fetch(self, mock_exists, mock_tmpf, mock_rmv):
         mock_exists.return_value = True
         with mock.patch('nova_powervm.virt.powervm.nvram.swift.open',
@@ -241,10 +237,10 @@ class TestSwiftStore(test.TestCase):
             self.assertRaises(api.NVRAMDownloadException,
                               self.swift_store.fetch, powervm.TEST_INST1)
 
-    @mock.patch('os.remove')
-    @mock.patch('tempfile.NamedTemporaryFile')
+    @mock.patch('os.remove', autospec=True)
+    @mock.patch('tempfile.NamedTemporaryFile', autospec=True)
     @mock.patch('nova_powervm.virt.powervm.nvram.swift.SwiftNvramStore.'
-                '_exists')
+                '_exists', autospec=True)
     def test_fetch_slot_map(self, mock_exists, mock_tmpf, mock_rmv):
         mock_exists.return_value = True
         with mock.patch('nova_powervm.virt.powervm.nvram.swift.open',
@@ -258,10 +254,10 @@ class TestSwiftStore(test.TestCase):
             self.assertEqual('data to read', data)
             mock_rmv.assert_called_once_with(m_open.return_value.name)
 
-    @mock.patch('os.remove')
-    @mock.patch('tempfile.NamedTemporaryFile')
+    @mock.patch('os.remove', autospec=True)
+    @mock.patch('tempfile.NamedTemporaryFile', autospec=True)
     @mock.patch('nova_powervm.virt.powervm.nvram.swift.SwiftNvramStore.'
-                '_exists')
+                '_exists', autospec=True)
     def test_fetch_slot_map_no_exist(self, mock_exists, mock_tmpf, mock_rmv):
         mock_exists.return_value = False
         data = self.swift_store.fetch_slot_map("test_slot")
@@ -298,7 +294,7 @@ class TestSwiftStore(test.TestCase):
                 'test_slot')
 
     @mock.patch('nova_powervm.virt.powervm.nvram.swift.SwiftNvramStore.'
-                '_get_object_names')
+                '_get_object_names', autospec=True)
     def test_exists(self, mock_get_obj_names):
         # Test where there are elements in here
         mock_get_obj_names.return_value = ['obj', 'obj1', 'obj2']
