@@ -1,4 +1,4 @@
-# Copyright 2014, 2016 IBM Corp.
+# Copyright 2014, 2017 IBM Corp.
 #
 # All Rights Reserved.
 #
@@ -24,17 +24,17 @@ from pypowervm.wrappers import event as pvm_evt
 from nova_powervm.virt.powervm import event
 
 
-class TestGetInstance(test.TestCase):
-    @mock.patch('nova.context.get_admin_context')
-    @mock.patch('nova_powervm.virt.powervm.vm.get_instance')
+class TestGetInstance(test.NoDBTestCase):
+    @mock.patch('nova.context.get_admin_context', autospec=True)
+    @mock.patch('nova_powervm.virt.powervm.vm.get_instance', autospec=True)
     def test_get_instance(self, mock_get_inst, mock_get_context):
         # If instance provided, vm.get_instance not called
         self.assertEqual('inst', event._get_instance('inst', 'uuid'))
-        mock_get_inst.assert_not_called()
+        self.assertEqual(0, mock_get_inst.call_count)
         # Note that we can only guarantee get_admin_context wasn't called
         # because _get_instance is mocked everywhere else in this suite.
         # Otherwise it could run from another test case executing in parallel.
-        mock_get_context.assert_not_called()
+        self.assertEqual(0, mock_get_context.call_count)
 
         # If instance not provided, vm.get_instance is called
         mock_get_inst.return_value = 'inst2'
@@ -48,7 +48,7 @@ class TestGetInstance(test.TestCase):
             # Don't reset mock_get_context
 
 
-class TestPowerVMNovaEventHandler(test.TestCase):
+class TestPowerVMNovaEventHandler(test.NoDBTestCase):
     def setUp(self):
         super(TestPowerVMNovaEventHandler, self).setUp()
         lceh_process_p = mock.patch(
@@ -59,14 +59,41 @@ class TestPowerVMNovaEventHandler(test.TestCase):
         self.mock_driver = mock.Mock()
         self.handler = event.PowerVMNovaEventHandler(self.mock_driver)
 
-    @mock.patch('nova_powervm.virt.powervm.event._get_instance')
+    @mock.patch('nova_powervm.virt.powervm.event._get_instance', autospec=True)
+    def test_get_inst_uuid(self, mock_get_instance):
+        fake_inst1 = mock.Mock(uuid='uuid1')
+        fake_inst2 = mock.Mock(uuid='uuid2')
+        mock_get_instance.side_effect = lambda i, u: {
+            'fake_pvm_uuid1': fake_inst1,
+            'fake_pvm_uuid2': fake_inst2}.get(u)
+
+        self.assertEqual(
+            (fake_inst1, 'uuid1'),
+            self.handler._get_inst_uuid(fake_inst1, 'fake_pvm_uuid1'))
+        self.assertEqual(
+            (fake_inst2, 'uuid2'),
+            self.handler._get_inst_uuid(fake_inst2, 'fake_pvm_uuid2'))
+        self.assertEqual(
+            (None, 'uuid1'),
+            self.handler._get_inst_uuid(None, 'fake_pvm_uuid1'))
+        self.assertEqual(
+            (fake_inst2, 'uuid2'),
+            self.handler._get_inst_uuid(fake_inst2, 'fake_pvm_uuid2'))
+        self.assertEqual(
+            (fake_inst1, 'uuid1'),
+            self.handler._get_inst_uuid(fake_inst1, 'fake_pvm_uuid1'))
+        mock_get_instance.assert_has_calls(
+            [mock.call(fake_inst1, 'fake_pvm_uuid1'),
+             mock.call(fake_inst2, 'fake_pvm_uuid2')])
+
+    @mock.patch('nova_powervm.virt.powervm.event._get_instance', autospec=True)
     def test_handle_inst_event(self, mock_get_instance):
         # If no event we care about, or NVRAM but no nvram_mgr, nothing happens
         self.mock_driver.nvram_mgr = None
         for dets in ([], ['foo', 'bar', 'baz'], ['NVRAM']):
             self.assertEqual('inst', self.handler._handle_inst_event(
                 'inst', 'uuid', dets))
-        mock_get_instance.assert_not_called()
+        self.assertEqual(0, mock_get_instance.call_count)
         self.mock_lceh_process.assert_not_called()
 
         self.mock_driver.nvram_mgr = mock.Mock()
@@ -74,7 +101,7 @@ class TestPowerVMNovaEventHandler(test.TestCase):
         # PartitionState only: no NVRAM handling, and inst is passed through.
         self.assertEqual('inst', self.handler._handle_inst_event(
             'inst', 'uuid', ['foo', 'PartitionState', 'bar']))
-        mock_get_instance.assert_not_called()
+        self.assertEqual(0, mock_get_instance.call_count)
         self.mock_driver.nvram_mgr.store.assert_not_called()
         self.mock_lceh_process.assert_called_once_with('inst', 'uuid')
 
@@ -89,24 +116,48 @@ class TestPowerVMNovaEventHandler(test.TestCase):
         self.mock_lceh_process.assert_not_called()
 
         mock_get_instance.reset_mock()
-        mock_get_instance.return_value = 'inst'
+        fake_inst = mock.Mock(uuid='fake-uuid')
+        mock_get_instance.return_value = fake_inst
 
         # NVRAM only - no PartitionState handling, instance is returned
-        self.assertEqual('inst', self.handler._handle_inst_event(
+        self.assertEqual(fake_inst, self.handler._handle_inst_event(
             None, 'uuid', ['NVRAM', 'baz']))
         mock_get_instance.assert_called_once_with(None, 'uuid')
-        self.mock_driver.nvram_mgr.store.assert_called_once_with('inst')
+        self.mock_driver.nvram_mgr.store.assert_called_once_with('fake-uuid')
         self.mock_lceh_process.assert_not_called()
 
         mock_get_instance.reset_mock()
         self.mock_driver.nvram_mgr.store.reset_mock()
+        self.handler._uuid_cache.clear()
 
         # Both event types
-        self.assertEqual('inst', self.handler._handle_inst_event(
+        self.assertEqual(fake_inst, self.handler._handle_inst_event(
             None, 'uuid', ['PartitionState', 'NVRAM']))
         mock_get_instance.assert_called_once_with(None, 'uuid')
-        self.mock_driver.nvram_mgr.store.assert_called_once_with('inst')
-        self.mock_lceh_process.assert_called_once_with('inst', 'uuid')
+        self.mock_driver.nvram_mgr.store.assert_called_once_with('fake-uuid')
+        self.mock_lceh_process.assert_called_once_with(fake_inst, 'uuid')
+
+        mock_get_instance.reset_mock()
+        self.mock_driver.nvram_mgr.store.reset_mock()
+        self.handler._uuid_cache.clear()
+
+        # Handle multiple NVRAM and PartitionState events
+        self.assertEqual(fake_inst, self.handler._handle_inst_event(
+            None, 'uuid', ['NVRAM']))
+        self.assertEqual(None, self.handler._handle_inst_event(
+            None, 'uuid', ['NVRAM']))
+        self.assertEqual(None, self.handler._handle_inst_event(
+            None, 'uuid', ['PartitionState']))
+        self.assertEqual(fake_inst, self.handler._handle_inst_event(
+            fake_inst, 'uuid', ['NVRAM']))
+        self.assertEqual(fake_inst, self.handler._handle_inst_event(
+            fake_inst, 'uuid', ['NVRAM', 'PartitionState']))
+        mock_get_instance.assert_called_once_with(None, 'uuid')
+        self.mock_driver.nvram_mgr.store.assert_has_calls(
+            [mock.call('fake-uuid')] * 4)
+        self.mock_lceh_process.assert_has_calls(
+            [mock.call(None, 'uuid'),
+             mock.call(fake_inst, 'uuid')])
 
     @mock.patch('nova_powervm.virt.powervm.event.PowerVMNovaEventHandler.'
                 '_handle_inst_event')
@@ -167,17 +218,80 @@ class TestPowerVMNovaEventHandler(test.TestCase):
              # inst1 pulled from the cache based on uuid1
              mock.call('inst1', 'uuid1', ['blah', 'PartitionState'])])
 
+    @mock.patch('nova_powervm.virt.powervm.event._get_instance', autospec=True)
+    @mock.patch('pypowervm.util.get_req_path_uuid', autospec=True)
+    def test_uuid_cache(self, mock_get_rpu, mock_get_instance):
+        deluri = pvm_evt.EventType.DELETE_URI
+        moduri = pvm_evt.EventType.MODIFY_URI
 
-class TestPowerVMLifecycleEventHandler(test.TestCase):
+        fake_inst1 = mock.Mock(uuid='uuid1')
+        fake_inst2 = mock.Mock(uuid='uuid2')
+        fake_inst4 = mock.Mock(uuid='uuid4')
+        mock_get_instance.side_effect = lambda i, u: {
+            'fake_pvm_uuid1': fake_inst1,
+            'fake_pvm_uuid2': fake_inst2,
+            'fake_pvm_uuid4': fake_inst4}.get(u)
+        mock_get_rpu.side_effect = lambda d, **k: d.split('/')[1]
+
+        uuid_det = (('fake_pvm_uuid1', 'NVRAM', moduri),
+                    ('fake_pvm_uuid2', 'NVRAM', moduri),
+                    ('fake_pvm_uuid4', 'NVRAM', moduri),
+                    ('fake_pvm_uuid1', 'NVRAM', moduri),
+                    ('fake_pvm_uuid2', '', deluri),
+                    ('fake_pvm_uuid2', 'NVRAM', moduri),
+                    ('fake_pvm_uuid1', '', deluri),
+                    ('fake_pvm_uuid3', '', deluri))
+        events = [
+            mock.Mock(etype=etype, data='LogicalPartition/' + uuid,
+                      detail=detail) for uuid, detail, etype in uuid_det]
+        self.handler.process(events[0:4])
+        mock_get_instance.assert_has_calls([
+            mock.call(None, 'fake_pvm_uuid1'),
+            mock.call(None, 'fake_pvm_uuid2'),
+            mock.call(None, 'fake_pvm_uuid4')])
+        self.assertEqual({
+            'fake_pvm_uuid1': 'uuid1',
+            'fake_pvm_uuid2': 'uuid2',
+            'fake_pvm_uuid4': 'uuid4'}, self.handler._uuid_cache)
+
+        mock_get_instance.reset_mock()
+
+        # Test the cache with a second process call
+        self.handler.process(events[4:7])
+        mock_get_instance.assert_has_calls([
+            mock.call(None, 'fake_pvm_uuid2')])
+        self.assertEqual({
+            'fake_pvm_uuid2': 'uuid2',
+            'fake_pvm_uuid4': 'uuid4'}, self.handler._uuid_cache)
+
+        mock_get_instance.reset_mock()
+
+        # Make sure a delete to a non-cached UUID doesn't blow up
+        self.handler.process([events[7]])
+        self.assertEqual(0, mock_get_instance.call_count)
+
+        mock_get_rpu.reset_mock()
+        mock_get_instance.reset_mock()
+
+        clear_events = [mock.Mock(etype=pvm_evt.EventType.NEW_CLIENT),
+                        mock.Mock(etype=pvm_evt.EventType.CACHE_CLEARED)]
+        # This should clear the cache
+        self.handler.process(clear_events)
+        self.assertEqual(dict(), self.handler._uuid_cache)
+        self.assertEqual(0, mock_get_rpu.call_count)
+        self.assertEqual(0, mock_get_instance.call_count)
+
+
+class TestPowerVMLifecycleEventHandler(test.NoDBTestCase):
     def setUp(self):
         super(TestPowerVMLifecycleEventHandler, self).setUp()
         self.mock_driver = mock.MagicMock()
         self.handler = event.PowerVMLifecycleEventHandler(self.mock_driver)
 
-    @mock.patch('nova_powervm.virt.powervm.vm.get_vm_qp')
-    @mock.patch('nova_powervm.virt.powervm.event._get_instance')
-    @mock.patch('nova_powervm.virt.powervm.vm.translate_event')
-    @mock.patch('nova.virt.event.LifecycleEvent')
+    @mock.patch('nova_powervm.virt.powervm.vm.get_vm_qp', autospec=True)
+    @mock.patch('nova_powervm.virt.powervm.event._get_instance', autospec=True)
+    @mock.patch('nova_powervm.virt.powervm.vm.translate_event', autospec=True)
+    @mock.patch('nova.virt.event.LifecycleEvent', autospec=True)
     def test_emit_event(self, mock_lce, mock_tx_evt, mock_get_inst, mock_qp):
         def assert_qp():
             mock_qp.assert_called_once_with(
@@ -192,9 +306,9 @@ class TestPowerVMLifecycleEventHandler(test.TestCase):
         mock_qp.side_effect = exception.InstanceNotFound(instance_id='uuid')
         self.handler._emit_event('uuid', None)
         assert_qp()
-        mock_get_inst.assert_not_called()
-        mock_tx_evt.assert_not_called()
-        mock_lce.assert_not_called()
+        self.assertEqual(0, mock_get_inst.call_count)
+        self.assertEqual(0, mock_tx_evt.call_count)
+        self.assertEqual(0, mock_lce.call_count)
         self.mock_driver.emit_event.assert_not_called()
 
         # Let get_vm_qp return its usual mock from now on
@@ -205,8 +319,8 @@ class TestPowerVMLifecycleEventHandler(test.TestCase):
         self.handler._emit_event('uuid', 'inst')
         assert_qp()
         assert_get_inst()
-        mock_tx_evt.assert_not_called()
-        mock_lce.assert_not_called()
+        self.assertEqual(0, mock_tx_evt.call_count)
+        self.assertEqual(0, mock_lce.call_count)
         self.mock_driver.emit_event.assert_not_called()
 
         # Ignore if task_state isn't one we care about
@@ -215,8 +329,8 @@ class TestPowerVMLifecycleEventHandler(test.TestCase):
             self.handler._emit_event('uuid', 'inst')
             assert_qp()
             assert_get_inst()
-            mock_tx_evt.assert_not_called()
-            mock_lce.assert_not_called()
+            self.assertEqual(0, mock_tx_evt.call_count)
+            self.assertEqual(0, mock_lce.call_count)
             self.mock_driver.emit_event.assert_not_called()
 
         # Task state we care about from now on
@@ -252,7 +366,7 @@ class TestPowerVMLifecycleEventHandler(test.TestCase):
         self.assertEqual({'uuid2': 'thread2'},
                          self.handler._delayed_event_threads)
 
-    @mock.patch('eventlet.greenthread.spawn_after')
+    @mock.patch('eventlet.greenthread.spawn_after', autospec=True)
     def test_process(self, mock_spawn):
         thread1 = mock.Mock()
         thread2 = mock.Mock()
