@@ -1,5 +1,5 @@
 # Copyright 2013 OpenStack Foundation
-# Copyright 2015, 2016 IBM Corp.
+# Copyright IBM Corp. and contributors
 #
 # All Rights Reserved.
 #
@@ -18,6 +18,7 @@
 import abc
 
 import oslo_log.log as logging
+from oslo_utils import excutils
 from oslo_utils import units
 import random
 import six
@@ -30,7 +31,6 @@ import pypowervm.wrappers.virtual_io_server as pvm_vios
 
 from nova_powervm.virt.powervm import exception as npvmex
 from nova_powervm.virt.powervm.i18n import _
-from nova_powervm.virt.powervm.i18n import _LW
 from nova_powervm.virt.powervm import mgmt
 from nova_powervm.virt.powervm import vm
 
@@ -73,6 +73,8 @@ class DiskAdapter(object):
 
     capabilities = {
         'shared_storage': False,
+        'has_imagecache': False,
+        'snapshot': False,
     }
 
     def __init__(self, adapter, host_uuid):
@@ -85,7 +87,7 @@ class DiskAdapter(object):
         self.host_uuid = host_uuid
         self.mp_uuid = mgmt.mgmt_uuid(self.adapter)
 
-    @property
+    @abc.abstractproperty
     def vios_uuids(self):
         """List the UUIDs of the Virtual I/O Servers hosting the storage."""
         raise NotImplementedError()
@@ -105,6 +107,16 @@ class DiskAdapter(object):
         """
         return {}
 
+    def manage_image_cache(self, context, all_instances):
+        """Update the image cache.
+
+        Only called if implentation has the capability: has_imagecache.
+
+        :param context: nova context
+        :param all_instances: List of all instances on the node
+        """
+        pass
+
     def validate(self, disk_info):
         """Validate the disk information is compatible with this driver.
 
@@ -117,41 +129,27 @@ class DiskAdapter(object):
         return _('The configured disk driver does not support migration '
                  'or resize.')
 
-    def disk_match_func(self, disk_type, instance):
+    @abc.abstractmethod
+    def _disk_match_func(self, disk_type, instance):
         """Return a matching function to locate the disk for an instance.
 
         :param disk_type: One of the DiskType enum values.
         :param instance: The instance whose disk is to be found.
         :return: Callable suitable for the match_func parameter of the
-                 pypowervm.tasks.scsi_mapper.find_maps method, with the
-                 following specification:
-            def match_func(storage_elem)
-                param storage_elem: A backing storage element wrapper (VOpt,
-                                    VDisk, PV, or LU) to be analyzed.
-                return: True if the storage_elem's mapping should be included;
-                        False otherwise.
+                 pypowervm.tasks.scsi_mapper.find_maps method.
         """
         raise NotImplementedError()
 
-    def boot_disk_path_for_instance(self, instance, vios_uuid):
-        """Find scsi mappings on given vios for the instance.
-
-        This method finds all scsi mappings on a given vios that are associated
-        with the instance and disk_type.
+    def get_bootdisk_path(self, instance, vios_uuid):
+        """Find the local path for the instance's boot disk.
 
         :param instance: nova.objects.instance.Instance object owning the
                          requested disk.
-        :param disk_type: The type of disk to find, one of the DiskType enum
-                          values.
-        :param lpar_wrap: pypowervm.wrappers.logical_partition.LPAR
-                          corresponding to the instance.  If not specified, it
-                          will be retrieved; i.e. specify this parameter to
-                          save on REST calls.
-        :return: Iterator of scsi mappings that are associated with the
-                 instance and disk_type.
+        :param vios_uuid: PowerVM UUID of the VIOS to search for mappings.
+        :return: Local path for instance's boot disk.
         """
         vm_uuid = vm.get_pvm_uuid(instance)
-        match_func = self.disk_match_func(DiskType.BOOT, instance)
+        match_func = self._disk_match_func(DiskType.BOOT, instance)
         vios_wrap = pvm_vios.VIOS.get(self.adapter, uuid=vios_uuid,
                                       xag=[pvm_const.XAG.VIO_SMAP])
         maps = tsk_map.find_maps(vios_wrap.scsi_mappings,
@@ -160,27 +158,20 @@ class DiskAdapter(object):
             return maps[0].server_adapter.backing_dev_name
         return None
 
-    def instance_disk_iter(self, instance, disk_type=DiskType.BOOT,
-                           lpar_wrap=None):
-        """Return the instance's storage element wrapper of the specified type.
+    def _get_bootdisk_iter(self, instance):
+        """Return an iterator of (storage_elem, VIOS) tuples for the instance.
+
+        This method returns an iterator of (storage_elem, VIOS) tuples, where
+        storage_elem is a pypowervm storage element wrapper associated with
+        the instance boot disk and VIOS is the wrapper of the Virtual I/O
+        server owning that storage element.
 
         :param instance: nova.objects.instance.Instance object owning the
                          requested disk.
-        :param disk_type: The type of disk to find, one of the DiskType enum
-                          values.
-        :param lpar_wrap: pypowervm.wrappers.logical_partition.LPAR
-                          corresponding to the instance.  If not specified, it
-                          will be retrieved; i.e. specify this parameter to
-                          save on REST calls.
-        :return: Iterator of tuples of (storage_elem, VIOS), where storage_elem
-                 is a storage element wrapper (pypowervm.wrappers.storage.VOpt,
-                 VDisk, PV, or LU) associated with the instance; and VIOS is
-                 the wrapper of the Virtual I/O Server owning that storage
-                 element.
+        :return: Iterator of tuples of (storage_elem, VIOS).
         """
-        if lpar_wrap is None:
-            lpar_wrap = vm.get_instance_wrapper(self.adapter, instance)
-        match_func = self.disk_match_func(disk_type, instance)
+        lpar_wrap = vm.get_instance_wrapper(self.adapter, instance)
+        match_func = self._disk_match_func(DiskType.BOOT, instance)
         for vios_uuid in self.vios_uuids:
             vios_wrap = pvm_vios.VIOS.get(
                 self.adapter, uuid=vios_uuid, xag=[pvm_const.XAG.VIO_SMAP])
@@ -198,12 +189,8 @@ class DiskAdapter(object):
                       made.
         :raise InstanceDiskMappingFailed: If the mapping could not be done.
         """
-        msg_args = {'instance_name': instance.name}
-        lpar_wrap = vm.get_instance_wrapper(self.adapter, instance)
-        for stg_elem, vios in self.instance_disk_iter(instance,
-                                                      lpar_wrap=lpar_wrap):
-            msg_args['disk_name'] = stg_elem.name
-            msg_args['vios_name'] = vios.name
+        for stg_elem, vios in self._get_bootdisk_iter(instance):
+            msg_args = {'disk_name': stg_elem.name, 'vios_name': vios.name}
 
             # Create a new mapping.  NOTE: If there's an existing mapping on
             # the other VIOS but not this one, we'll create a second mapping
@@ -212,24 +199,23 @@ class DiskAdapter(object):
             # alternative would be always checking all VIOSes for existing
             # mappings, which increases the response time of the common case by
             # an entire GET of VIOS+VIO_SMAP.
-            LOG.debug("Mapping boot disk %(disk_name)s of instance "
-                      "%(instance_name)s to the management partition from "
-                      "Virtual I/O Server %(vios_name)s.", msg_args)
+            LOG.debug("Mapping boot disk %(disk_name)s to the management "
+                      "partition from Virtual I/O Server %(vios_name)s.",
+                      msg_args, instance=instance)
             try:
                 tsk_map.add_vscsi_mapping(self.host_uuid, vios, self.mp_uuid,
                                           stg_elem)
                 # If that worked, we're done.  add_vscsi_mapping logged.
                 return stg_elem, vios
-            except Exception as e:
-                msg_args['exc'] = e
-                LOG.warning(_LW("Failed to map boot disk %(disk_name)s of "
-                                "instance %(instance_name)s to the management "
-                                "partition from Virtual I/O Server "
-                                "%(vios_name)s: %(exc)s"), msg_args)
+            except Exception:
+                LOG.exception("Failed to map boot disk %(disk_name)s to the "
+                              "management partition from Virtual I/O Server "
+                              "%(vios_name)s.", msg_args, instance=instance)
                 # Try the next hit, if available.
         # We either didn't find the boot dev, or failed all attempts to map it.
-        raise npvmex.InstanceDiskMappingFailed(**msg_args)
+        raise npvmex.InstanceDiskMappingFailed(instance_name=instance.name)
 
+    @abc.abstractmethod
     def disconnect_disk_from_mgmt(self, vios_uuid, disk_name):
         """Disconnect a disk from the management partition.
 
@@ -239,21 +225,15 @@ class DiskAdapter(object):
         """
         raise NotImplementedError()
 
-    @property
+    @abc.abstractproperty
     def capacity(self):
-        """Capacity of the storage in gigabytes
+        """Capacity of the storage in gigabytes."""
+        raise NotImplementedError()
 
-        Default is to make the capacity arbitrarily large
-        """
-        return 1 << 21
-
-    @property
+    @abc.abstractproperty
     def capacity_used(self):
-        """Capacity of the storage in gigabytes that is used
-
-        Default is to say none of it is used.
-        """
-        return 0
+        """Capacity of the storage in gigabytes that is used."""
+        raise NotImplementedError()
 
     @staticmethod
     def _get_disk_name(disk_type, instance, short=False):
@@ -273,9 +253,30 @@ class DiskAdapter(object):
             base, prefix=prefix, max_len=pvm_const.MaxLen.VDISK_NAME if short
             else pvm_const.MaxLen.FILENAME_DEFAULT)
 
+    @staticmethod
+    def get_name_by_uuid(disk_type, uuid, short=False):
+        """Generate a name for a DiskType using a given uuid.
+
+        :param disk_type: One of the DiskType enum values.
+        :param uuid: The uuid to use for the name
+        :param short: If True the generate name will be limited to 15
+                      characters.  If False it will be limited by the API.
+        :return: A name base off of disk_type and uuid.
+        """
+        prefix = '%s_' % (disk_type[0] if short else disk_type)
+        return pvm_util.sanitize_file_name_for_api(
+            uuid, prefix=prefix, max_len=pvm_const.MaxLen.VDISK_NAME if short
+            else pvm_const.MaxLen.FILENAME_DEFAULT)
+
     @staticmethod
     def _get_image_name(image_meta, max_len=pvm_const.MaxLen.FILENAME_DEFAULT):
-        """Generate a name for a virtual storage copy of an image."""
+        """Generate a name for a virtual storage copy of an image.
+
+        :param nova.objects.ImageMeta image_meta:
+            The metadata of the image of the instance.
+        :param max_len: Maximum string length for the resulting image name.
+        :return: String name for the image on the server.
+        """
         return pvm_util.sanitize_file_name_for_api(
             image_meta.name, prefix=DiskType.IMAGE + '_',
             suffix='_' + image_meta.checksum, max_len=max_len)
@@ -296,11 +297,10 @@ class DiskAdapter(object):
                 disk_bytes = floor
         return disk_bytes
 
-    def disconnect_image_disk(self, context, instance, stg_ftsk=None,
-                              disk_type=None):
+    @abc.abstractmethod
+    def disconnect_disk(self, instance, stg_ftsk=None, disk_type=None):
         """Disconnects the storage adapters from the image disk.
 
-        :param context: nova context for operation
         :param instance: instance to disconnect the image for.
         :param stg_ftsk: (Optional) The pypowervm transaction FeedTask for the
                          I/O Operations.  If provided, the Virtual I/O Server
@@ -313,20 +313,19 @@ class DiskAdapter(object):
         :return: A list of all the backing storage elements that were
                  disconnected from the I/O Server and VM.
         """
-        pass
+        raise NotImplementedError()
 
-    def delete_disks(self, context, instance, storage_elems):
+    @abc.abstractmethod
+    def delete_disks(self, storage_elems):
         """Removes the disks specified by the mappings.
 
-        :param context: nova context for operation
-        :param instance: instance to delete the disk for.
         :param storage_elems: A list of the storage elements that are to be
                               deleted.  Derived from the return value from
-                              disconnect_image_disk.
+                              disconnect_disk.
         """
-        pass
+        raise NotImplementedError()
 
-    def create_disk_from_image(self, context, instance, image_meta, disk_size,
+    def create_disk_from_image(self, context, instance, image_meta,
                                image_type=DiskType.BOOT):
         """Creates a disk and copies the specified image to it.
 
@@ -334,10 +333,6 @@ class DiskAdapter(object):
         :param instance: instance to create the disk for.
         :param nova.objects.ImageMeta image_meta:
             The metadata of the image of the instance.
-        :param disk_size: The size of the disk to create in GB.  If smaller
-                          than the image, it will be ignored (as the disk
-                          must be at least as big as the image).  Must be an
-                          int.
         :param image_type: the image type. See disk constants above.
         :return: The backing pypowervm storage object that was created.
         """
@@ -346,21 +341,19 @@ class DiskAdapter(object):
         for attempt in range(1, 5):
             try:
                 return self._create_disk_from_image(
-                    context, instance, image_meta,
-                    disk_size, image_type=image_type)
-            except Exception as error:
-                if attempt < 4:
-                    LOG.exception(error)
-                    LOG.warning(_LW("Instance %(inst)s Disk Upload attempt "
-                                    "#%(attempt)d failed. Retrying the "
-                                    "upload."),
-                                {"attempt": attempt,
-                                "inst": instance.name}, instance=instance)
-                    time.sleep(random.randint(1, 5))
-                else:
-                    raise
+                    context, instance, image_meta, image_type=image_type)
+            except Exception:
+                with excutils.save_and_reraise_exception(
+                        logger=LOG, reraise=False) as sare:
+                    if attempt < 4:
+                        LOG.exception("Disk Upload attempt #%d failed. "
+                                      "Retrying the upload.", attempt,
+                                      instance=instance)
+                        time.sleep(random.randint(1, 5))
+                    else:
+                        sare.reraise = True
 
-    def _create_disk_from_image(self, context, instance, image_meta, disk_size,
+    def _create_disk_from_image(self, context, instance, image_meta,
                                 image_type=DiskType.BOOT):
         """Creates a disk and copies the specified image to it.
 
@@ -370,19 +363,15 @@ class DiskAdapter(object):
         :param instance: instance to create the disk for.
         :param nova.objects.ImageMeta image_meta:
             The metadata of the image of the instance.
-        :param disk_size: The size of the disk to create in GB.  If smaller
-                          than the image, it will be ignored (as the disk
-                          must be at least as big as the image).  Must be an
-                          int.
         :param image_type: the image type. See disk constants above.
         :return: The backing pypowervm storage object that was created.
         """
         pass
 
-    def connect_disk(self, context, instance, disk_info, stg_ftsk=None):
+    @abc.abstractmethod
+    def connect_disk(self, instance, disk_info, stg_ftsk=None):
         """Connects the disk image to the Virtual Machine.
 
-        :param context: nova context for the transaction.
         :param instance: nova instance to connect the disk to.
         :param disk_info: The pypowervm storage element returned from
                           create_disk_from_image.  Ex. VOptMedia, VDisk, LU,
@@ -394,18 +383,19 @@ class DiskAdapter(object):
                          the FeedTask is not provided, the updates will be run
                          immediately when this method is executed.
         """
-        pass
+        raise NotImplementedError()
 
-    def extend_disk(self, context, instance, disk_info, size):
+    @abc.abstractmethod
+    def extend_disk(self, instance, disk_info, size):
         """Extends the disk.
 
-        :param context: nova context for operation.
         :param instance: instance to extend the disk for.
         :param disk_info: dictionary with disk info.
         :param size: the new size in gb.
         """
         raise NotImplementedError()
 
+    @abc.abstractmethod
     def check_instance_shared_storage_local(self, context, instance):
         """Check if instance files located on shared storage.
 
@@ -417,6 +407,7 @@ class DiskAdapter(object):
         """
         raise NotImplementedError()
 
+    @abc.abstractmethod
     def check_instance_shared_storage_remote(self, context, data):
         """Check if instance files located on shared storage.
 
