@@ -1,5 +1,5 @@
 # Copyright 2013 OpenStack Foundation
-# Copyright 2015, 2017 IBM Corp.
+# Copyright IBM Corp. and contributors
 #
 # All Rights Reserved.
 #
@@ -48,6 +48,7 @@ class LocalStorage(disk_dvr.DiskAdapter):
     capabilities = {
         'shared_storage': False,
         'has_imagecache': True,
+        # NOTE(efried): 'snapshot' capability set dynamically in __init__.
     }
 
     def __init__(self, adapter, host_uuid):
@@ -65,6 +66,12 @@ class LocalStorage(disk_dvr.DiskAdapter):
         self.image_cache_mgr = imagecache.ImageManager(self._vios_uuid,
                                                        self.vg_uuid, adapter)
         self.cache_lock = lockutils.ReaderWriterLock()
+        # Set the 'snapshot' capability dynamically.  If we're hosting I/O on
+        # the management partition, we can snapshot.  If we're hosting I/O on
+        # traditional VIOS, we are limited by the fact that a VSCSI device
+        # can't be mapped to two partitions (the VIOS and the management) at
+        # once.
+        self.capabilities['snapshot'] = self.mp_uuid == self._vios_uuid
         LOG.info("Local Storage driver initialized: volume group: '%s'",
                  self.vg_name)
 
@@ -76,21 +83,17 @@ class LocalStorage(disk_dvr.DiskAdapter):
         """
         return [self._vios_uuid]
 
-    def disk_match_func(self, disk_type, instance):
+    @staticmethod
+    def _disk_match_func(disk_type, instance):
         """Return a matching function to locate the disk for an instance.
 
         :param disk_type: One of the DiskType enum values.
         :param instance: The instance whose disk is to be found.
         :return: Callable suitable for the match_func parameter of the
-                 pypowervm.tasks.scsi_mapper.find_maps method, with the
-                 following specification:
-            def match_func(storage_elem)
-                param storage_elem: A backing storage element wrapper (VOpt,
-                                    VDisk, PV, or LU) to be analyzed.
-                return: True if the storage_elem's mapping should be included;
-                        False otherwise.
+                 pypowervm.tasks.scsi_mapper.find_maps method.
         """
-        disk_name = self._get_disk_name(disk_type, instance, short=True)
+        disk_name = LocalStorage._get_disk_name(disk_type, instance,
+                                                short=True)
         return tsk_map.gen_match_func(pvm_stg.VDisk, names=[disk_name])
 
     @property
@@ -336,6 +339,25 @@ class LocalStorage(disk_dvr.DiskAdapter):
                 LOG.exception("PowerVM Error extending disk.",
                               instance=instance)
 
+    def check_instance_shared_storage_local(self, context, instance):
+        """Check if instance files located on shared storage.
+
+        This runs check on the destination host, and then calls
+        back to the source host to check the results.
+
+        :param context: security context
+        :param instance: nova.objects.instance.Instance object
+        """
+        raise NotImplementedError()
+
+    def check_instance_shared_storage_remote(self, context, data):
+        """Check if instance files located on shared storage.
+
+        :param context: security context
+        :param data: result of check_instance_shared_storage_local
+        """
+        raise NotImplementedError()
+
     def _get_vg_wrap(self):
         return pvm_stg.VG.get(self.adapter, uuid=self.vg_uuid,
                               parent_type=pvm_vios.VIOS,
