@@ -26,6 +26,7 @@ from nova import objects
 from nova import utils as n_utils
 from nova.virt import configdrive
 from nova.virt import driver
+import os_resource_classes as orc
 from oslo_log import log as logging
 from oslo_utils import importutils
 import six
@@ -93,7 +94,6 @@ class PowerVMDriver(driver.ComputeDriver):
     def __init__(self, virtapi):
         self.capabilities = {
             # NOTE(edmondsw): 'has_imagecache' will be set dynamically
-            "supports_recreate": True,
             "supports_migrate_to_same_host": False,
             "supports_attach_interface": True,
             "supports_device_tagging": False,
@@ -101,6 +101,7 @@ class PowerVMDriver(driver.ComputeDriver):
             "supports_tagged_attach_volume": False,
             "supports_extend_volume": True,
             "supports_multiattach": False,
+            "supports_evacuate": True,
         }
         super(PowerVMDriver, self).__init__(virtapi)
 
@@ -226,10 +227,11 @@ class PowerVMDriver(driver.ComputeDriver):
                  {'op': op, 'display_name': instance.display_name,
                   'name': instance.name}, instance=instance)
 
-    def get_info(self, instance):
+    def get_info(self, instance, use_cache=True):
         """Get the current status of an instance.
 
         :param instance: nova.objects.instance.Instance object
+        :param use_cache: unused in this driver
         :returns: An InstanceInfo object
         """
         return vm.get_vm_info(self.adapter, instance)
@@ -297,12 +299,13 @@ class PowerVMDriver(driver.ComputeDriver):
     def get_host_cpu_stats(self):
         """Return the current CPU state of the host."""
         self.host_cpu_cache.refresh()
+        total_cycles = self.host_cpu_cache.total_cycles
+        total_user_cycles = self.host_cpu_cache.total_user_cycles
+        total_fw_cycles = self.host_cpu_cache.total_fw_cycles
         return {
             'kernel': self.host_cpu_cache.total_fw_cycles,
             'user': self.host_cpu_cache.total_user_cycles,
-            'idle': (self.host_cpu_cache.total_cycles -
-                     self.host_cpu_cache.total_user_cycles -
-                     self.host_cpu_cache.total_fw_cycles),
+            'idle': (total_cycles - total_user_cycles - total_fw_cycles),
             # Not reported by PowerVM
             'iowait': 0,
             'frequency': self.host_cpu_cache.cpu_freq}
@@ -382,8 +385,9 @@ class PowerVMDriver(driver.ComputeDriver):
         flow_spawn = tf_lf.Flow("spawn")
 
         # Determine if this is a VM recreate
-        recreate = (instance.task_state == task_states.REBUILD_SPAWNING and
-                    'id' not in image_meta)
+        task_state = instance.task_state
+        rebuild_spawning = task_states.REBUILD_SPAWNING
+        recreate = (task_state == rebuild_spawning and 'id' not in image_meta)
 
         # Create the transaction manager (FeedTask) for Storage I/O.
         xag = self._get_inst_xag(instance, bdms, recreate=recreate)
@@ -399,9 +403,11 @@ class PowerVMDriver(driver.ComputeDriver):
             vol_drv_iter=vol_drv_iter)
 
         # Create the LPAR, check if NVRAM restore is needed.
-        nvram_mgr = (self.nvram_mgr if self.nvram_mgr and
-                     (recreate or instance.vm_state in FETCH_NVRAM_STATES)
-                     else None)
+        vm_state = instance.vm_state
+        if self.nvram_mgr and (recreate or vm_state in FETCH_NVRAM_STATES):
+            nvram_mgr = self.nvram_mgr
+        else:
+            nvram_mgr = None
 
         # If we're recreating pass None in for the FeedTask. This will make the
         # Create task produce a FeedTask that will be used to scrub stale
@@ -462,11 +468,7 @@ class PowerVMDriver(driver.ComputeDriver):
 
         pwr_opts = pvm_popts.PowerOnOpts()
         if CONF.powervm.remove_vopt_media_on_boot:
-            # Get the cfgdrive media name for the vopt removal task in PowerOn.
-            media_name = pvm_util.sanitize_file_name_for_api(
-                instance.name, prefix=media.CFG_DRV_PREFIX,
-                suffix=media.CFG_DRV_SUFFIX,
-                max_len=pvm_const.MaxLen.VOPT_NAME)
+            media_name = media.ConfigDrivePowerVM.get_cfg_drv_name(instance)
             pwr_opts.remove_optical(
                 media_name, time=CONF.powervm.remove_vopt_media_time)
 
@@ -552,7 +554,6 @@ class PowerVMDriver(driver.ComputeDriver):
 
     def _destroy(self, context, instance, block_device_info=None,
                  network_info=None, destroy_disks=True, shutdown=True):
-
         """Internal destroy method used by multiple operations.
 
         :param context: security context
@@ -654,14 +655,13 @@ class PowerVMDriver(driver.ComputeDriver):
             # the prior step.
             flow.add(tf_vm.Delete(self.adapter, instance))
 
-            if (destroy_disks and
-                    instance.vm_state not in KEEP_NVRAM_STATES and
-                    instance.host in [None, CONF.host]):
-                # If the disks are being destroyed and not one of the
-                # operations that we should keep the NVRAM around for, then
-                # it's probably safe to delete the NVRAM from the store.
-                flow.add(tf_vm.DeleteNvram(self.nvram_mgr, instance))
-                flow.add(tf_slot.DeleteSlotStore(instance, slot_mgr))
+            if (destroy_disks and instance.vm_state not in KEEP_NVRAM_STATES):
+                if instance.host in [None, CONF.host]:
+                    # If the disks are being destroyed and not one of the
+                    # operations that we should keep the NVRAM around for, then
+                    # it's probably safe to delete the NVRAM from the store.
+                    flow.add(tf_vm.DeleteNvram(self.nvram_mgr, instance))
+                    flow.add(tf_slot.DeleteSlotStore(instance, slot_mgr))
 
             # Run the flow
             tf_base.run(flow, instance=instance)
@@ -674,11 +674,11 @@ class PowerVMDriver(driver.ComputeDriver):
                         instance=instance)
             return
         except Exception as e:
-                LOG.exception("PowerVM error destroying instance.",
-                              instance=instance)
-                # Convert to a Nova exception
-                raise exception.InstanceTerminationFailure(
-                    reason=six.text_type(e))
+            LOG.exception("PowerVM error destroying instance.",
+                          instance=instance)
+            # Convert to a Nova exception
+            raise exception.InstanceTerminationFailure(
+                reason=six.text_type(e))
 
     def destroy(self, context, instance, network_info, block_device_info=None,
                 destroy_disks=True, migrate_data=None):
@@ -753,12 +753,14 @@ class PowerVMDriver(driver.ComputeDriver):
         # to revise in the future as volume connectors evolve.
         instance.save()
 
-    def extend_volume(self, connection_info, instance):
+    def extend_volume(self, connection_info, instance, requested_size):
         """Extend the disk attached to the instance.
 
         :param dict connection_info: The connection for the extended volume.
         :param nova.objects.instance.Instance instance:
             The instance whose volume gets extended.
+        :param int requested_size: The requested new volume size in bytes. This
+            parameter is unused by this driver.
         :return: None
         """
         vol_drv = vol_attach.build_volume_driver(
@@ -974,6 +976,9 @@ class PowerVMDriver(driver.ComputeDriver):
         """
         # Do this here so it refreshes each time this method is called.
         self.host_wrapper = pvm_ms.System.get(self.adapter)[0]
+        return self._get_available_resource()
+
+    def _get_available_resource(self):
         # Get host information
         data = pvm_host.build_host_resource_from_ms(self.host_wrapper)
 
@@ -983,6 +988,62 @@ class PowerVMDriver(driver.ComputeDriver):
 
         return data
 
+    def update_provider_tree(self, provider_tree, nodename, allocations=None):
+        """Update a ProviderTree with current provider and inventory data.
+
+        :param nova.compute.provider_tree.ProviderTree provider_tree:
+            A nova.compute.provider_tree.ProviderTree object representing all
+            the providers in the tree associated with the compute node, and any
+            sharing providers (those with the ``MISC_SHARES_VIA_AGGREGATE``
+            trait) associated via aggregate with any of those providers (but
+            not *their* tree- or aggregate-associated providers), as currently
+            known by placement.
+        :param nodename:
+            String name of the compute node (i.e.
+            ComputeNode.hypervisor_hostname) for which the caller is requesting
+            updated provider information.
+        :param allocations: Currently ignored by this driver.
+        """
+        # Get (legacy) resource information. Same as get_available_resource,
+        # but we don't need to refresh self.host_wrapper as it was *just*
+        # refreshed by get_available_resource in the resource tracker's
+        # update_available_resource flow.
+        data = self._get_available_resource()
+
+        # TODO(efried): Fix these to reflect something like reality
+        # For now, duplicate the logic the resource tracker uses via
+        # update_compute_node when get_inventory/update_provider_tree is not
+        # implemented.
+        cpu_alloc_ratio = CONF.cpu_allocation_ratio or 16.0
+        cpu_reserved = CONF.reserved_host_cpus
+        mem_alloc_ratio = CONF.ram_allocation_ratio or 1.5
+        mem_reserved = CONF.reserved_host_memory_mb
+        disk_alloc_ratio = CONF.disk_allocation_ratio or 1.0
+        disk_reserved = self._get_reserved_host_disk_gb_from_config()
+
+        inventory = {
+            orc.VCPU: {
+                'total': data['vcpus'],
+                'max_unit': data['vcpus'],
+                'allocation_ratio': cpu_alloc_ratio,
+                'reserved': cpu_reserved,
+            },
+            orc.MEMORY_MB: {
+                'total': data['memory_mb'],
+                'max_unit': data['memory_mb'],
+                'allocation_ratio': mem_alloc_ratio,
+                'reserved': mem_reserved,
+            },
+            orc.DISK_GB: {
+                # TODO(efried): Proper DISK_GB sharing when SSP driver in play
+                'total': int(data['local_gb']),
+                'max_unit': int(data['local_gb']),
+                'allocation_ratio': disk_alloc_ratio,
+                'reserved': disk_reserved,
+            },
+        }
+        provider_tree.update_inventory(nodename, inventory)
+
     def get_host_uptime(self):
         """Returns the result of calling "uptime" on the target host."""
         # trivial implementation from libvirt/driver.py for consistency
@@ -1146,7 +1207,7 @@ class PowerVMDriver(driver.ComputeDriver):
             # If VM is moving to a new host make sure the NVRAM is at the very
             # latest.
             flow.add(tf_vm.StoreNvram(self.nvram_mgr, instance,
-                     immediate=True))
+                                      immediate=True))
         if flavor.root_gb > instance.root_gb:
             # Resize the root disk
             flow.add(tf_stg.ExtendDisk(
@@ -1233,8 +1294,8 @@ class PowerVMDriver(driver.ComputeDriver):
             self._log_operation('finish migration', instance)
 
         # Ensure the disk drivers are compatible.
-        if (not same_host and
-                not self._is_booted_from_volume(block_device_info)):
+        booted_from_vol = self._is_booted_from_volume(block_device_info)
+        if (not same_host and not booted_from_vol):
             # Can't migrate the disks if they are not on shared storage
             if not self.disk_dvr.capabilities['shared_storage']:
                 raise exception.InstanceFaultRollback(
