@@ -1,4 +1,4 @@
-# Copyright 2015 IBM Corp.
+# Copyright 2015, 2017 IBM Corp.
 #
 # All Rights Reserved.
 #
@@ -101,6 +101,11 @@ class VolumeAttachFailed(nex.NovaException):
                 "machine %(instance_name)s.  %(reason)s")
 
 
+class VolumeExtendFailed(nex.NovaException):
+    msg_fmt = _("Unable to extend volume (id: %(volume_id)s) on virtual "
+                "machine %(instance_name)s.")
+
+
 class VolumeDetachFailed(nex.NovaException):
     msg_fmt = _("Unable to detach volume (id: %(volume_id)s) from virtual "
                 "machine %(instance_name)s.  %(reason)s")
