@@ -1,4 +1,4 @@
-# Copyright 2016 IBM Corp.
+# Copyright 2016, 2017 IBM Corp.
 #
 # All Rights Reserved.
 #
@@ -25,15 +25,16 @@ import types
 from nova_powervm import conf as cfg
 from nova_powervm.conf import powervm
 from nova_powervm.virt.powervm.i18n import _
-from nova_powervm.virt.powervm.i18n import _LI
-from nova_powervm.virt.powervm.i18n import _LW
 from nova_powervm.virt.powervm.nvram import api
 
 from oslo_concurrency import lockutils
 from oslo_log import log as logging
+from oslo_utils import excutils
+from oslo_utils import uuidutils
 from swiftclient import exceptions as swft_exc
 from swiftclient import service as swft_srv
 
+
 LOG = logging.getLogger(__name__)
 CONF = cfg.CONF
 
@@ -65,6 +66,7 @@ class SwiftNvramStore(api.NvramStore):
             'os_project_domain_name': CONF.powervm.swift_project_domain_name,
             'os_auth_url': CONF.powervm.swift_auth_url,
             'os_cacert': CONF.powervm.swift_cacert,
+            'os_endpoint_type': CONF.powervm.swift_endpoint_type,
         }
 
         return options
@@ -86,11 +88,11 @@ class SwiftNvramStore(api.NvramStore):
                     LOG.debug(str(r))
                 result = results
             else:
-                LOG.debug('SwiftOperation result: %s' % str(result))
+                LOG.debug('SwiftOperation result: %s', str(result))
             return result
-        except swft_srv.SwiftError as e:
-            LOG.exception(e)
-            raise
+        except swft_srv.SwiftError:
+            with excutils.save_and_reraise_exception(logger=LOG):
+                LOG.exception("Error running swift operation.")
 
     @classmethod
     def _get_name_from_listing(cls, results):
@@ -123,10 +125,10 @@ class SwiftNvramStore(api.NvramStore):
             container=container)
         return self._get_name_from_listing(results)
 
-    def _store(self, inst_key, inst_name, data, exists=None):
+    def _store(self, inst_key, data, exists=None):
         """Store the NVRAM into the storage service.
 
-        :param instance: instance object
+        :param inst_key: The key by which to store the data in the repository.
         :param data: the NVRAM data base64 encoded string
         :param exists: (Optional, Default: None) If specified, tells the upload
                        whether or not the object exists.  Should be a boolean
@@ -179,11 +181,11 @@ class SwiftNvramStore(api.NvramStore):
                         # If upload failed during nvram/slot_map update due to
                         # expired keystone token, retry swift-client operation
                         # to allow regeneration of token
-                        LOG.warning(_LW('NVRAM upload failed due to invalid '
-                                        'token. Retrying upload.'))
+                        LOG.warning('NVRAM upload failed due to invalid '
+                                    'token. Retrying upload.')
                         return True
                     # The upload failed.
-                    raise api.NVRAMUploadException(instance=inst_name,
+                    raise api.NVRAMUploadException(instance=inst_key,
                                                    reason=result)
             return False
         try:
@@ -192,23 +194,25 @@ class SwiftNvramStore(api.NvramStore):
             # The upload failed.
             reason = (_('Unable to store NVRAM after %d attempts') %
                       re.last_attempt.attempt_number)
-            raise api.NVRAMUploadException(instance=inst_name, reason=reason)
+            raise api.NVRAMUploadException(instance=inst_key, reason=reason)
 
     @lockutils.synchronized('nvram')
     def store(self, instance, data, force=True):
         """Store the NVRAM into the storage service.
 
-        :param instance: instance object
+        :param instance: The nova instance object OR instance UUID.
         :param data: the NVRAM data base64 encoded string
         :param force: boolean whether an update should always be saved,
                       otherwise, check to see if it's changed.
         """
-        exists = self._exists(instance.uuid)
+        inst_uuid = (instance if
+                     uuidutils.is_uuid_like(instance) else instance.uuid)
+        exists = self._exists(inst_uuid)
         if not force and exists:
             # See if the entry exists and has not changed.
             results = self._run_operation('stat', options={'long': True},
                                           container=self.container,
-                                          objects=[instance.uuid])
+                                          objects=[inst_uuid])
             result = results[0]
             if result['success']:
                 existing_hash = result['headers']['etag']
@@ -216,12 +220,12 @@ class SwiftNvramStore(api.NvramStore):
                     data = data.encode('ascii')
                 md5 = hashlib.md5(data).hexdigest()
                 if existing_hash == md5:
-                    LOG.info(_LI('NVRAM has not changed for instance: %s'),
-                             instance.name, instance=instance)
+                    LOG.info(('NVRAM has not changed for instance with '
+                             'UUID %s.'), inst_uuid)
                     return
 
-        self._store(instance.uuid, instance.name, data, exists=exists)
-        LOG.debug('NVRAM updated for instance: %s' % instance.name)
+        self._store(inst_uuid, data, exists=exists)
+        LOG.debug('NVRAM updated for instance with UUID %s', inst_uuid)
 
     def store_slot_map(self, inst_key, data):
         """Store the Slot Map to Swift.
@@ -229,7 +233,7 @@ class SwiftNvramStore(api.NvramStore):
         :param inst_key: The instance key to use for the storage operation.
         :param data: The data of the object to store.  This should be a string.
         """
-        self._store(inst_key, inst_key, data)
+        self._store(inst_key, data)
 
     def fetch_slot_map(self, inst_key):
         """Fetch the Slot Map object.
@@ -242,12 +246,14 @@ class SwiftNvramStore(api.NvramStore):
     def fetch(self, instance):
         """Fetch the NVRAM from the storage service.
 
-        :param instance: instance object
+        :param instance: The nova instance object or instance UUID.
         :returns: the NVRAM data base64 encoded string
         """
-        data, result = self._fetch(instance.uuid)
+        inst_uuid = (instance if
+                     uuidutils.is_uuid_like(instance) else instance.uuid)
+        data, result = self._fetch(inst_uuid)
         if not data:
-            raise api.NVRAMDownloadException(instance=instance.name,
+            raise api.NVRAMDownloadException(instance=inst_uuid,
                                              reason=result)
         return data
 
@@ -285,7 +291,7 @@ class SwiftNvramStore(api.NvramStore):
             try:
                 os.remove(f.name)
             except Exception:
-                LOG.warning(_LW('Could not remove temporary file: %s'), f.name)
+                LOG.warning('Could not remove temporary file: %s', f.name)
 
     def delete_slot_map(self, inst_key):
         """Delete the Slot Map from Swift.
@@ -295,7 +301,7 @@ class SwiftNvramStore(api.NvramStore):
         for result in self._run_operation('delete', container=self.container,
                                           objects=[inst_key]):
 
-            LOG.debug('Delete slot map result: %s' % str(result))
+            LOG.debug('Delete slot map result: %s', str(result))
             if not result['success']:
                 raise api.NVRAMDeleteException(reason=result,
                                                instance=inst_key)
@@ -303,12 +309,15 @@ class SwiftNvramStore(api.NvramStore):
     def delete(self, instance):
         """Delete the NVRAM into the storage service.
 
-        :param instance: instance object
+        :param instance: The nova instance object OR instance UUID.
         """
+        inst_uuid = (instance if
+                     uuidutils.is_uuid_like(instance) else instance.uuid)
         for result in self._run_operation('delete', container=self.container,
-                                          objects=[instance.uuid]):
+                                          objects=[inst_uuid]):
 
-            LOG.debug('Delete result: %s' % str(result), instance=instance)
+            LOG.debug('Delete result for instance with UUID %(inst_uuid)s: '
+                      '%(res)s', {'inst_uuid': inst_uuid, 'res': result})
             if not result['success']:
-                raise api.NVRAMDeleteException(instance=instance.name,
+                raise api.NVRAMDeleteException(instance=inst_uuid,
                                                reason=result)
