@@ -1,4 +1,4 @@
-# Copyright 2014, 2017 IBM Corp.
+# Copyright 2014, 2018 IBM Corp.
 #
 # All Rights Reserved.
 #
@@ -121,9 +121,9 @@ def translate_event(pvm_state, pwr_state):
         trans = event.EVENT_LIFECYCLE_STARTED
     elif pvm_state in STOPPED_EVENTS and pwr_state != power_state.SHUTDOWN:
         trans = event.EVENT_LIFECYCLE_STOPPED
-    elif (pvm_state in SUSPENDED_EVENTS and
-          pwr_state != power_state.SUSPENDED):
-        trans = event.EVENT_LIFECYCLE_SUSPENDED
+    elif pvm_state in SUSPENDED_EVENTS:
+        if pwr_state != power_state.SUSPENDED:
+            trans = event.EVENT_LIFECYCLE_SUSPENDED
     elif pvm_state in RESUMING_EVENTS and pwr_state != power_state.RUNNING:
         trans = event.EVENT_LIFECYCLE_RESUMED
 
@@ -151,80 +151,6 @@ def _translate_vm_state(pvm_state):
     return nova_state
 
 
-class InstanceInfo(hardware.InstanceInfo):
-    """Instance Information
-
-    This object tries to lazy load the attributes since the compute
-    manager retrieves it a lot just to check the status and doesn't need
-    all the attributes.
-
-    :param adapter: pypowervm adapter
-    :param name: instance name
-    :param uuid: powervm uuid
-    """
-    def __init__(self, adapter, name, uuid):
-        self._adapter = adapter
-        self._name = name
-        self._uuid = uuid
-        self._state = None
-        self._mem_kb = None
-        self._max_mem_kb = None
-        self._num_cpu = None
-        # Non-lazy loaded properties
-        self.cpu_time_ns = 0
-        # Todo(IBM) Should this be the instance UUID vs PowerVM?
-        self.id = uuid
-
-    def _get_property(self, q_prop):
-        return get_vm_qp(self._adapter, self._uuid, q_prop)
-
-    @property
-    def state(self):
-        # return the state if we previously loaded it
-        if self._state is not None:
-            return self._state
-
-        # otherwise, fetch the value now
-        pvm_state = self._get_property('PartitionState')
-        self._state = _translate_vm_state(pvm_state)
-        return self._state
-
-    @property
-    def mem_kb(self):
-        # return the memory if we previously loaded it
-        if self._mem_kb is not None:
-            return self._mem_kb
-
-        # otherwise, fetch the value now
-        pvm_mem_kb = self._get_property('CurrentMemory')
-        self._mem_kb = pvm_mem_kb
-        return self._mem_kb
-
-    @property
-    def max_mem_kb(self):
-        # return the max memory if we previously loaded it
-        if self._max_mem_kb is not None:
-            return self._max_mem_kb
-
-        # TODO(IBM) max isn't a quick property.  We need the wrapper
-        return self.mem_kb()
-
-    @property
-    def num_cpu(self):
-        # return the number of cpus if we previously loaded it
-        if self._num_cpu is not None:
-            return self._num_cpu
-
-        # otherwise, fetch the value now
-        pvm_num_cpu = self._get_property('AllocatedVirtualProcessors')
-        self._num_cpu = pvm_num_cpu
-        return self._num_cpu
-
-    def __eq__(self, other):
-        return (self.__class__ == other.__class__ and
-                self._uuid == other._uuid)
-
-
 class VMBuilder(object):
     """Converts a Nova Instance/Flavor into a pypowervm LPARBuilder."""
 
@@ -234,6 +160,8 @@ class VMBuilder(object):
     _PVM_SHAR_PROC_POOL = 'powervm:shared_proc_pool_name'
     _PVM_SRR_CAPABILITY = 'powervm:srr_capability'
     _PVM_PPT_RATIO = 'powervm:ppt_ratio'
+    _PVM_ENFORCE_AFFINITY_CHECK = 'powervm:enforce_affinity_check'
+    _PVM_SECURE_BOOT = 'powervm:secure_boot'
 
     # Map of PowerVM extra specs to the lpar builder attributes.
     # '' is used for attributes that are not implemented yet.
@@ -252,6 +180,8 @@ class VMBuilder(object):
         'powervm:availability_priority': lpar_bldr.AVAIL_PRIORITY,
         'powervm:enable_lpar_metric': lpar_bldr.ENABLE_LPAR_METRIC,
         _PVM_PPT_RATIO: lpar_bldr.PPT_RATIO,
+        _PVM_ENFORCE_AFFINITY_CHECK: lpar_bldr.ENFORCE_AFFINITY_CHECK,
+        _PVM_SECURE_BOOT: lpar_bldr.SECURE_BOOT,
         _PVM_UNCAPPED: None,
         _PVM_DED_SHAR_MODE: None,
         _PVM_PROC_COMPAT: None,
@@ -359,6 +289,18 @@ class VMBuilder(object):
                              "unsupported host.", instance=instance)
                 else:
                     attrs[bldr_key] = instance.flavor.extra_specs[key]
+            elif bldr_key == lpar_bldr.ENFORCE_AFFINITY_CHECK:
+                if (instance.task_state == task_states.REBUILD_SPAWNING and not
+                   self.host_w.get_capability('affinity_check_capable')):
+                    # We still want to be able to rebuild from hosts that
+                    # support affinity score check to hosts that don't support
+                    # affinity score check.
+                    LOG.info("Skipping affinity check attribute processing "
+                             "on rebuild to host which does not support "
+                             "affinity checks.", instance=instance)
+                else:
+                    attrs[bldr_key] = self._flavor_bool(
+                        instance.flavor.extra_specs[key], key)
             else:
                 # We found a direct mapping
                 attrs[bldr_key] = instance.flavor.extra_specs[key]
@@ -435,8 +377,8 @@ class VMBuilder(object):
         :param pool_name: The shared proc pool name.
         :return: The internal API id for the shared proc pool.
         """
-        if (pool_name is None or
-                pool_name == pvm_spp.DEFAULT_POOL_DISPLAY_NAME):
+        default_pool_name = pvm_spp.DEFAULT_POOL_DISPLAY_NAME
+        if (pool_name is None or pool_name == default_pool_name):
             # The default pool is 0
             return 0
 
@@ -790,12 +732,7 @@ def get_pvm_uuid(instance):
                      uuid.
     :return: pvm_uuid.
     """
-    # NOTE(esberglu): To work around bug ##1766692, we explicitly use str()
-    # rather than six.text_type here because of this pypowervm check for
-    # isinstance(..., str) at L50 of
-    # https://github.com/powervm/pypowervm/blob/1.1.10/pypowervm/utils/uuid.py
-    inst_uuid = (instance if uuidutils.is_uuid_like(instance)
-                 else str(instance.uuid))
+    inst_uuid = instance if uuidutils.is_uuid_like(instance) else instance.uuid
     return pvm_uuid.convert_uuid_to_pvm(inst_uuid).upper()
 
 
