@@ -19,6 +19,7 @@ import os
 import six
 from taskflow import task
 
+from nova import exception as nova_exc
 from nova_powervm import conf as cfg
 from nova_powervm.virt.powervm import exception as p_exc
 from nova_powervm.virt.powervm import vm
@@ -93,10 +94,10 @@ class FileIOVolumeAdapter(v_driver.PowerVMVolumeAdapter):
 
     def _connect_volume(self, slot_mgr):
         path = self._get_path()
-        # Get the File Path
+        volid = self.connection_info['data']['volume_id']
         fio = pvm_stg.FileIO.bld(
             self.adapter, path,
-            backstore_type=pvm_stg.BackStoreType.FILE_IO)
+            backstore_type=pvm_stg.BackStoreType.LOOP, tag=volid)
 
         def add_func(vios_w):
             # If the vios doesn't match, just return
@@ -136,6 +137,12 @@ class FileIOVolumeAdapter(v_driver.PowerVMVolumeAdapter):
         self.stg_ftsk.add_post_execute(task.FunctorTask(
             set_slot_info, name='file_io_slot_%s' % path))
 
+    def extend_volume(self):
+        path = self._get_path()
+        if path is None:
+            raise nova_exc.InvalidBDM()
+        self._extend_volume(path)
+
     def _disconnect_volume(self, slot_mgr):
         # Build the match function
         match_func = tsk_map.gen_match_func(pvm_stg.VDisk,
