@@ -1,4 +1,4 @@
-# Copyright 2015, 2017 IBM Corp.
+# Copyright 2015, 2018 IBM Corp.
 #
 # All Rights Reserved.
 #
@@ -77,6 +77,11 @@ class NPIVVolumeAdapter(v_driver.FibreChannelVolumeAdapter):
         for fabric in self._fabric_names():
             self._add_maps_for_fabric(fabric, slot_mgr)
 
+    def extend_volume(self):
+        # The compute node does not need to take any additional steps for the
+        # client to see the extended volume.
+        pass
+
     def _disconnect_volume(self, slot_mgr):
         """Disconnect the volume.
 
@@ -90,10 +95,11 @@ class NPIVVolumeAdapter(v_driver.FibreChannelVolumeAdapter):
         # of the spawn. We also want to check that the instance is on this
         # host. If it isn't then we can remove the mappings because this is
         # being called as the result of an evacuation clean up.
-        if (self.instance.task_state not in TASK_STATES_FOR_DISCONNECT and
-           self.instance.host in [None, CONF.host]):
-            # NPIV should only remove the VFC mapping upon a destroy of the VM
-            return
+        if (self.instance.task_state not in TASK_STATES_FOR_DISCONNECT):
+            if (self.instance.host in [None, CONF.host]):
+                # NPIV should only remove the VFC mapping upon a destroy of
+                # the VM
+                return
 
         # Run the disconnect for each fabric
         for fabric in self._fabric_names():
@@ -331,8 +337,7 @@ class NPIVVolumeAdapter(v_driver.FibreChannelVolumeAdapter):
         :return: True if the instance appears to be migrating to this host.
                  False otherwise.
         """
-        return (fc_state == FS_INST_MAPPED and
-                self.instance.host != CONF.host)
+        return fc_state == FS_INST_MAPPED and self.instance.host != CONF.host
 
     def _configure_wwpns_for_migration(self, fabric):
         """Configures the WWPNs for a migration.
